@@ -41,6 +41,7 @@ void back_to_stars(char *aligned, char *ori);
 int save_fasta_file_no_gaponly(char *fname, char **sequence, int tot_seqs, int *each_length, int *sel_seqs);
 void unalign(SEA_VIEW *view);
 int reorder_seqs(int tot, char **seq, char **name);
+int check_alignment_characters(SEA_VIEW *view, int *sel_seqs);
 
 /* extern proto */
 extern int run_external_prog_in_pseudoterm(char *cmd, const char *dialogfname, const char *label);
@@ -106,6 +107,7 @@ for(num = 0; num < tot_seqs; num++) {
 			}
 		line[l_line++] = sequence[num][pos];
 		if(sequence[num][pos] == '*') line[l_line - 1] = 'X';
+		if(sequence[num][pos] == '?') line[l_line - 1] = 'X';
 		}
 	line[l_line] = 0;
 	seqlen += l_line;
@@ -184,6 +186,7 @@ if(my_align_choice == ALIGN_ALL)
 	memset(tmpselseqs, 1, view->tot_seqs * sizeof(int));
 else
 	memcpy(tmpselseqs, view->sel_seqs, view->tot_seqs * sizeof(int));
+if ( check_alignment_characters(view, tmpselseqs) ) return;
 status = save_part_as_pir(debut, fin, view->sequence, view->seqname, 
 	tmpselseqs, view->tot_seqs, fname, &withU, &num_longest, view->protein);
 //if(view->protein) //clustalw2 now accepts U and muscle also
@@ -462,6 +465,7 @@ for(oldseq = 0; oldseq < view->tot_seqs; oldseq++) {
 		while (*q == '-') q++;
 		if(*p == 0 || *q == 0) break;
 		if( *p == '*' && *q == 'X') *q = '*';
+		if( *p == '?' && *q == 'X') *q = '?';
 		if( toupper(*p) != toupper(*q) ) return TRUE;
 		if( islower(*p) ) *q = *p;
 		}
@@ -937,7 +941,8 @@ if(count < 2) {
 }
 
 enum {profile_clustalw2, profile_muscle};
-static int num_sp_set, use_quicktree, profile_to_profile = FALSE, profile_algo = profile_clustalw2;
+static int num_sp_set, use_quicktree, profile_to_profile = FALSE, profile_algo = profile_clustalw2,
+  do_propagate;
 int profile_dialog(SEA_VIEW *view)
 // returns TRUE iff cancel
 {
@@ -976,13 +981,18 @@ int profile_dialog(SEA_VIEW *view)
 	quicktree->type(FL_TOGGLE_BUTTON);
 	if(profile_algo == profile_clustalw2) quicktree->value(1);
 	else quicktree->deactivate();
+  Fl_Check_Button *propagate = new Fl_Check_Button(quicktree->x() + quicktree->w(), y, 170, 20, 
+						   "Propagate gaps to sites");
+  propagate->type(FL_TOGGLE_BUTTON);
+  if(view->regions == NULL && view->tot_comment_lines == 0) propagate->deactivate();
+  else propagate->value(1);
 	y += 25;
 	Fl_Button *cancel_button = new Fl_Button(5, y, 60, 20, "Cancel");
 	cancel_button->shortcut(myFL_CTRL | 'w');
 	Fl_Button *help_button = new Fl_Button(97, y, 60, 20, "Help");
 	Fl_Button *ok_button = new Fl_Return_Button(190, y, 60, 20, "OK");
 	w->end();
-	w->set_modal();
+	w->set_non_modal();
 	w->show();
 	while (TRUE) {
 		Fl_Widget *o = Fl::readqueue();
@@ -990,8 +1000,8 @@ int profile_dialog(SEA_VIEW *view)
 		else if(o == cancel_button || o == w ) { retval = 1; break; }
 		else if(o == ok_button ){ retval = 0; break; }
 		else if(o == help_button ) { 
-			direct_help_callback(NULL, (void *)"Align Menu");
-			}
+		  direct_help_callback(o, (void *)"Align Menu");
+		  }
 		else if(o == choice_algo) {
 			profile_algo = choice_algo->value();
 			if(profile_algo == profile_muscle) {
@@ -1009,6 +1019,7 @@ int profile_dialog(SEA_VIEW *view)
 		}
 	num_sp_set = choice->value();
 	use_quicktree = quicktree->value();
+  do_propagate = propagate->value();
 	profile_to_profile = profile->value();
 	delete w;
 	return retval;
@@ -1041,6 +1052,7 @@ void profile_align(SEA_VIEW *view)
 			return;
 			}
 		}
+	if ( check_alignment_characters(view, view->list_species_sets[num_sp_set]) ) return;
 	status = save_fasta_file_no_gaponly(fname1, view->sequence, view->tot_seqs, view->each_length, 
 							 view->list_species_sets[num_sp_set]);
 	if(status) {
@@ -1048,15 +1060,16 @@ void profile_align(SEA_VIEW *view)
 		remove(fname1);
 		return;
 		}
-	if(view->protein) replace_stars(fname1);
+	replace_stars(fname1);
 	sprintf(fname2, "%s_2.fasta", base_fname);
+	if ( check_alignment_characters(view, view->sel_seqs) ) return;
 	if(profile_to_profile) {
 		status = save_fasta_file_no_gaponly(fname2, view->sequence, view->tot_seqs, view->each_length, 
 											view->sel_seqs);
-		if(view->protein) replace_stars(fname2);
 		}
 	else status = save_part_as_pir(1, view->seq_length, view->sequence, view->seqname, 
 					view->sel_seqs, view->tot_seqs, fname2, &withU, &num_longest, view->protein);
+	replace_stars(fname2);
 	if(status) {
 		fl_alert("Cannot write sequences to filename\n%s", fname2);
 		remove(fname2);
@@ -1122,14 +1135,38 @@ void profile_align(SEA_VIEW *view)
 		if(  ! view->list_species_sets[num_sp_set][num] ) continue;
 		rang++;
 		table[rang] = num;
-		if(view->protein) back_to_stars(seq[rang], view->sequence[num]);
+		back_to_stars(seq[rang], view->sequence[num]);
 	}
 	for(num = 0; num < view->tot_seqs; num++) {
 		if(  ! view->sel_seqs[num] ) continue;
 		rang++;
 		table[rang] = num;
-		if(view->protein) back_to_stars(seq[rang], view->sequence[num]);
+		back_to_stars(seq[rang], view->sequence[num]);
 	}
+  if(do_propagate && (view->regions != NULL || view->tot_comment_lines > 0) ) {
+    //propagate gaps created in profile seqs to regions and comment lines
+    int l = 0, num_align = table[0];
+    char *p, *q;
+    p = view->sequence[num_align]; // one profile seq
+    q = seq[0]; // this seq after profile alignment
+    while(*p != 0) {
+      while(*p != 0 && *p == *q) {
+	p++; q++;
+	}
+      if(*p == 0 && *q == 0) break;
+      if(*q == '-') {
+	l = 1;
+	q++;
+	while(*q != *p) { q++; l++; }
+	}
+      if(l > 0) {
+	//insert gap of length l in regions
+	insert_region_part(view, p - view->sequence[num_align], l);
+	if(view->tot_comment_lines > 0) insert_gap_all_comments(l, p - view->sequence[num_align], view);
+	l = 0;
+      }
+    }
+  }
 	//replace seaview seqs by those from the file
 	if(view->numb_gc > 1) {
 		calc_color_function = ( view->protein ? get_color_for_aa : get_color_for_base );
@@ -1158,7 +1195,7 @@ void profile_align(SEA_VIEW *view)
 
 
 void replace_stars(char *fname)
-// replace all * in file by X
+// replace all * and ? in file by X
 {
 	FILE *in = fopen(fname, "rb+");
 	if(in == NULL) return;
@@ -1170,6 +1207,8 @@ void replace_stars(char *fname)
 	all[l] = 0;
 	char *p = all;
 	while((p = strchr(p, '*')) != NULL) *p = 'X';
+	p = all;
+	while((p = strchr(p, '?')) != NULL) *p = 'X';
 	fseek(in, 0, SEEK_SET);
 	fwrite(all, 1, l, in);
 	fclose(in);
@@ -1178,12 +1217,14 @@ void replace_stars(char *fname)
 
 void back_to_stars(char *aligned, char *ori)
 // replaces X in aligned by * when * at same place in ori
+// replaces X in aligned by ? when ? at same place in ori
 {
 	while (TRUE) {
 		while(*aligned == '-') aligned++;
 		while(*ori == '-') ori++;
 		if(*ori == 0 || *aligned == 0) break;
 		if(*aligned == 'X' && *ori == '*') *aligned = '*';
+		if(*aligned == 'X' && *ori == '?') *aligned = '?';
 		aligned++;
 		ori++;
 		}
@@ -1304,3 +1345,46 @@ int reorder_seqs(int tot, char **seq, char **name)
   free(newseq);
   return FALSE;
 }
+
+
+int check_alignment_characters(SEA_VIEW *view, int *sel_seqs)
+//returns TRUE iff unexpected characters are present and alignment shd be cancelled
+//otherwise, replaces unexpected characters by X or N and returns FALSE
+{
+  int i, choice;
+  char *p, X;
+  const char dna_accepted[] = "ABCDGHKMNRSTUVWXY-*?";
+  const char prot_accepted[] = "ABCDEFGHIKLMNPQRSTVWXYZ-*?";
+  const char *accepted;
+  char unexpected[50] = "", single[2] = "";
+  
+  X = view->protein ? 'X' : 'N';
+  accepted = view->protein ? prot_accepted : dna_accepted;
+  for(i = 0; i < view->tot_seqs; i++) {
+    if(!sel_seqs[i]) continue;
+    p = view->sequence[i] - 1;
+    while(*++p) {
+      if(strchr(accepted, toupper(*p)) == NULL) {
+	if(strlen(unexpected) + 1 < sizeof(unexpected) && strchr(unexpected, *p) == NULL) {
+	  *single = *p;
+	  strcat(unexpected, single);
+	}
+      }
+    }
+  }
+  if(*unexpected == 0) return FALSE;
+  choice = fl_choice("Unexpected characters are present in sequence data '%s'.\n"
+		     "You can replace them by '%c' and continue aligning, or cancel alignment:", 
+		     "Replace && align", "Cancel", NULL, unexpected, view->protein ? 'X' : 'N');
+  if(choice) return TRUE;
+  for(i = 0; i < view->tot_seqs; i++) {
+    if(!sel_seqs[i]) continue;
+    p = view->sequence[i] - 1;
+    while(*++p) {
+      if(strchr(accepted, toupper(*p)) == NULL) *p = X;
+      }
+    }
+  set_seaview_modified(view, TRUE);
+  return FALSE;
+}
+
