@@ -767,7 +767,10 @@ char calc_site_consensus(int *freqs, int dernier, int threshold, char *residues,
 			kind = num;
 		}
 	}
-	if(!protein) total -= freqs[16];//DNA: don't count uninformative N's for frequency calculation
+	if(!protein) {
+	  total -= freqs[16];//DNA: don't count uninformative N's for frequency calculation
+	  if(total == 0) return 'N';
+	  }
 	float minval = total * (threshold / 100.);
 	if(iupac && !protein ) { // for DNA/RNA with IUPAC codes
 		if( maxi >= minval ) retval = residues[kind - 1];
