@@ -1,6 +1,9 @@
 #ifdef __APPLE__
 #include <Carbon/Carbon.h>
 #import <Cocoa/Cocoa.h>
+#if ! (defined(__LP64__) && __LP64__)
+#include <QuickTime/QuickTimeComponents.h>
+#endif
 #ifndef NSINTEGER_DEFINED //appears with 10.5 in NSObjCRuntime.h
 #if defined(__LP64__) && __LP64__
 typedef long NSInteger;
@@ -72,7 +75,7 @@ static int MG_doing_copy = 0;
 
 void fl_ringbell(int a)
 {
-	AlertSoundPlay();
+  fl_beep(FL_BEEP_DEFAULT);
 }
 
 
@@ -598,10 +601,7 @@ void MyCopyPictToClipboard (PicHandle mypicture)
 }
 #endif
 
-//next 2 functions to copy quartz data to clipboard both as pdf and as PICT (or jpeg for 64-bit) bitmap
-#if MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_4
-#include <QuickTime/QuickTimeComponents.h>
-#endif
+//next 2 functions copy quartz data to clipboard both as pdf and as PICT (or tiff for 64-bit) bitmap
 void *prepare_copy_pdf_and_pict(int w, int h)
 {
   Fl_X::q_release_context();
@@ -609,12 +609,13 @@ void *prepare_copy_pdf_and_pict(int w, int h)
   CGRect bounds = CGRectMake(0, 0, w, h );	
   CFMutableDataRef pdfdata = CFDataCreateMutable(NULL, 0);
   CGDataConsumerRef myconsumer;
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4
-  myconsumer = CGDataConsumerCreateWithCFData (pdfdata);
-#else
-  static CGDataConsumerCallbacks callbacks = { MyPutBytes, NULL };
-  myconsumer = CGDataConsumerCreate ((void*) pdfdata, &callbacks);
-#endif
+  if(CGDataConsumerCreateWithCFData != NULL) { // true from 10.4
+    myconsumer = CGDataConsumerCreateWithCFData (pdfdata);
+    }
+  else {
+    static CGDataConsumerCallbacks callbacks = { MyPutBytes, NULL };
+    myconsumer = CGDataConsumerCreate ((void*) pdfdata, &callbacks);
+    }
   fl_gc = CGPDFContextCreate (myconsumer, &bounds, NULL);
   CGDataConsumerRelease (myconsumer);
   if (fl_gc == NULL) return NULL;
@@ -629,9 +630,6 @@ void complete_copy_pdf_and_pict(void *data)
 {
   int w, h;
   CFMutableDataRef pdfdata = (CFMutableDataRef)data;
-  CGRect clip = CGContextGetClipBoundingBox(fl_gc);
-  w = (int)(clip.size.width + 0.5);
-  h = (int)(clip.size.height + 0.5);
   CGContextEndPage (fl_gc);
   CGContextRelease (fl_gc);
   PasteboardRef clipboard = NULL;
@@ -643,14 +641,26 @@ void complete_copy_pdf_and_pict(void *data)
   
   
   //second, transform this PDF to a bitmap image and put it as PICT/jpeg in clipboard
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4
-  CGDataProviderRef prov = CGDataProviderCreateWithCFData(pdfdata);
-#else
-  CGDataProviderRef prov = CGDataProviderCreateWithData(NULL, CFDataGetBytePtr(pdfdata), CFDataGetLength(pdfdata), NULL);
-#endif
+  CGDataProviderRef prov;
+  if(CGDataProviderCreateWithCFData != NULL) { // true from 10.4
+    prov = CGDataProviderCreateWithCFData(pdfdata);
+    }
+  else {
+    prov = CGDataProviderCreateWithData(NULL, CFDataGetBytePtr(pdfdata), CFDataGetLength(pdfdata), NULL);
+  }
   CGPDFDocumentRef pdfdoc = CGPDFDocumentCreateWithProvider(prov);
+  CGPDFPageRef pdfpage = CGPDFDocumentGetPage(pdfdoc, 1);
+  CGRect clip = CGPDFPageGetBoxRect(pdfpage, kCGPDFCropBox);
+  w = (int)(clip.size.width + 0.5);
+  h = (int)(clip.size.height + 0.5);
   CGDataProviderRelease(prov);
-  CGColorSpaceRef space = CGColorSpaceCreateWithName(/*kCGColorSpaceGenericRGB*/ kCGColorSpaceUserRGB);
+  CGColorSpaceRef space = CGColorSpaceCreateWithName(
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4
+						     kCGColorSpaceGenericRGB
+#else
+						     kCGColorSpaceUserRGB
+#endif
+						     );
   const int scale = 2;
   w *= scale; h *= scale;
   void *mem = malloc(w * h * sizeof(int));
@@ -663,49 +673,50 @@ void complete_copy_pdf_and_pict(void *data)
   CGContextDrawPDFDocument(fl_gc, rect, pdfdoc, 1);
   CGPDFDocumentRelease(pdfdoc);
   CFRelease(pdfdata);
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4
-  //other way for >= 10.4 that doesn't use QuickTime:
-  CFMutableDataRef pictdata = CFDataCreateMutable(NULL, 0);
+  if(CGBitmapContextCreateImage != NULL) {// way for >= 10.4 that doesn't use QuickTime:
+    CFMutableDataRef pictdata = CFDataCreateMutable(NULL, 0);
 #if __LP64__
-  //CFStringRef  uti = CFSTR("public.jpeg");
-  CFStringRef  uti = CFSTR("public.tiff");
+    CFStringRef  uti = CFSTR("public.tiff");
 #else
-  CFStringRef  uti = CFSTR("com.apple.pict");
+    CFStringRef  uti = CFSTR("com.apple.pict");
 #endif
-  CGImageRef image = CGBitmapContextCreateImage(fl_gc);
-  CGImageDestinationRef dest = CGImageDestinationCreateWithData(pictdata, uti, 1, NULL);
-  CGImageDestinationAddImage(dest, image, NULL);
-  CGImageDestinationFinalize(dest);
-  CGImageRelease(image);
-  CFRelease(dest);
+    CGImageRef image = CGBitmapContextCreateImage(fl_gc);
+    CGImageDestinationRef dest = CGImageDestinationCreateWithData(pictdata, uti, 1, NULL);
+    CGImageDestinationAddImage(dest, image, NULL);
+    CGImageDestinationFinalize(dest);
+    CGImageRelease(image);
+    CFRelease(dest);
 #if !__LP64__
-  CFRange range = CFRangeMake(0, 512);
-  CFDataDeleteBytes(pictdata, range);// The PICT clipboard REQUIRES only the image data, not the header.
+    CFRange range = CFRangeMake(0, 512);
+    CFDataDeleteBytes(pictdata, range);// The PICT clipboard REQUIRES only the image data, not the header.
 #endif
-  PasteboardPutItemFlavor(clipboard, (PasteboardItemID)1, uti, pictdata, kPasteboardFlavorNoFlags );
-  CFRelease (pictdata);    
-#else
-  GraphicsExportComponent exporter;
-  OSErr err = OpenADefaultComponent(GraphicsExporterComponentType, kQTFileTypePicture /*'PICT'*/, &exporter);
-  err = GraphicsExportSetInputCGBitmapContext(exporter, fl_gc);
-  Handle dataHandle = NewHandle(0);
-  err = GraphicsExportSetOutputHandle(exporter, dataHandle);
-  unsigned long size;
-  err = GraphicsExportDoExport(exporter, &size);
-  err = CloseComponent(exporter); 
-  if(GetHandleSize(dataHandle) > 512) {
-    HLock(dataHandle);
-    // The clipboard REQUIRES only the image data, not the header.
-    CFDataRef pictdata = CFDataCreate(NULL, (const UInt8 *)*dataHandle + 512, GetHandleSize(dataHandle) - 512);
-    HUnlock(dataHandle);
-    PasteboardPutItemFlavor(clipboard, (PasteboardItemID)1, 
-						    CFSTR("com.apple.pict"),
-						    pictdata,
-						    kPasteboardFlavorNoFlags ); 
+    PasteboardPutItemFlavor(clipboard, (PasteboardItemID)1, uti, pictdata, kPasteboardFlavorNoFlags );
     CFRelease (pictdata);    
-  }
-  DisposeHandle(dataHandle);
+    }
+  else {// way for < 10.4 that uses QuickTime:
+#if !__LP64__
+    GraphicsExportComponent exporter;
+    OSErr err = OpenADefaultComponent(GraphicsExporterComponentType, kQTFileTypePicture /*'PICT'*/, &exporter);
+    err = GraphicsExportSetInputCGBitmapContext(exporter, fl_gc);
+    Handle dataHandle = NewHandle(0);
+    err = GraphicsExportSetOutputHandle(exporter, dataHandle);
+    unsigned long size;
+    err = GraphicsExportDoExport(exporter, &size);
+    err = CloseComponent(exporter); 
+    if(GetHandleSize(dataHandle) > 512) {
+      HLock(dataHandle);
+      // The clipboard REQUIRES only the image data, not the header.
+      CFDataRef pictdata = CFDataCreate(NULL, (const UInt8 *)*dataHandle + 512, GetHandleSize(dataHandle) - 512);
+      HUnlock(dataHandle);
+      PasteboardPutItemFlavor(clipboard, (PasteboardItemID)1, 
+						      CFSTR("com.apple.pict"),
+						      pictdata,
+						      kPasteboardFlavorNoFlags ); 
+      CFRelease (pictdata);    
+    }
+    DisposeHandle(dataHandle);
 #endif
+  }
   CFRelease (clipboard);    
   CGContextRelease(fl_gc);
   fl_gc = NULL;
