@@ -5,8 +5,9 @@
 
 enum position_mask {custom_mask = -1, position_1, position_2, position_3, position_12,
   variable_sites };
-const char *position_mask_names[5] = {"First codon pos.", "Second codon pos.",
+const char *position_mask_names[] = {"First codon pos.", "Second codon pos.",
 "Third codon pos.", "1st+2nd codon pos.", "Variable sites"};
+int position_mask_count = sizeof(position_mask_names)/sizeof(char*);
 
 /* prototypes of included functions */
 region *duplicate_region(region *source, const char *name);
@@ -82,6 +83,7 @@ while(pseg != NULL) {
 	prev = dest_seg;
 	pseg = pseg->next;
 	}
+destin->hide_when_viewasprots = source->hide_when_viewasprots;
 return destin;
 }
 
@@ -207,6 +209,7 @@ int create_codon_position_mask(SEA_VIEW *view, position_mask mask, const char *r
   maregion->name = (char *)malloc(strlen(regname) + 1);
   if(maregion->name ==  NULL) out_of_memory();
   strcpy(maregion->name, regname);
+  if(!view->protein) maregion->hide_when_viewasprots = TRUE;
   
   memset(view->region_line, '-', view->seq_length);
   if(mask < variable_sites) {
@@ -369,6 +372,7 @@ list_segments *segment;
 list = view->regions;
 while ( --rang ) 
 	list = list->next;
+if(view->viewasprots && list->element->hide_when_viewasprots) return;
 if(view->active_region == NULL) {
 	Fl_Menu_Item *items = 
 		(Fl_Menu_Item *) ((Fl_Menu_ *)view->menu_file)->menu();
@@ -739,7 +743,7 @@ view->region_length -= total;
 
 void delete_region_part(SEA_VIEW *view, int numsite, int total)
 {
-int debut, fin;
+int debut, fin, vnumsite, vtotal;
 list_segments *segment, *previous;
 list_regions *next_region;
 list_regions active;
@@ -756,13 +760,21 @@ if(view->active_region != NULL) {
 else
 	next_region = view->regions;
 while(next_region != NULL) {
+	if(view->viewasprots && next_region->element->hide_when_viewasprots) {
+	  vtotal = total * 3;
+	  vnumsite = numsite * 3;
+	}
+	else {
+	  vtotal = total;
+	  vnumsite = numsite;
+	}
 	segment = next_region->element->list;
 	while(segment != NULL) {
 		debut = segment->debut; fin = segment->fin;
-		if(debut >= numsite) debut -= total;
-		else if(debut >= numsite - total) debut = numsite - total;
-		if(fin >= numsite - total) {
-			fin -= total;
+		if(debut >= vnumsite) debut -= vtotal;
+		else if(debut >= vnumsite - vtotal) debut = vnumsite - vtotal;
+		if(fin >= vnumsite - vtotal) {
+			fin -= vtotal;
 			if(fin < debut) {
 				debut = segment->debut;
 				segment = segment->next;
@@ -828,6 +840,7 @@ void insert_region_part(SEA_VIEW *view, int numsite, int total)
 {
 list_segments *segment;
 list_regions *next_region;
+int vtotal, vnumsite;
 
 if(total + view->region_length > view->max_seq_length)
 	total = view->max_seq_length - view->region_length;
@@ -836,9 +849,17 @@ if(view->active_region != NULL) insert_in_active_region(view, numsite, total);
 next_region = view->regions;
 while(next_region != NULL) {
 	segment = next_region->element->list;
+	if(view->viewasprots && next_region->element->hide_when_viewasprots) {
+	  vtotal = total * 3;
+	  vnumsite = numsite * 3;
+	  }
+	else {
+	  vtotal = total;
+	  vnumsite = numsite;
+	  }
 	while(segment != NULL) {
-		if(segment->debut > numsite) segment->debut += total;
-		if(segment->fin >= numsite) segment->fin += total;
+		if(segment->debut > vnumsite) segment->debut += vtotal;
+		if(segment->fin >= vnumsite) segment->fin += vtotal;
 		segment = segment->next;
 		}
 	next_region = next_region->next;
