@@ -1,3 +1,6 @@
+#ifndef WIN32
+#include <sys/times.h>
+#endif
 #include "seaview.h"
 #include "treedraw.h"
 #include <FL/Fl_Round_Button.H>
@@ -20,6 +23,7 @@ typedef enum {observed_ndist, Jukes_Cantor, Kimura2P, HKY, LogDet, Ka, Ks} nucle
 allseq *view_to_allseq(SEA_VIEW *view, int remove_all_gaps);
 void gaps_as_unknown_states(allseq *phyml_seqs, int protein);
 void free_after_view_to_allseq(allseq *phyml_seqs);
+void initialize_random_series(void);
 void bootstrap_weights(allseq *seqs, int use_codons);
 void savedistinfile(matrix *mat, const char *dirname);
 matrix *calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwlseqs, int in_bootstrap);
@@ -881,6 +885,12 @@ void run_phyml_callback(Fl_Widget *ob, void *data)
   remove(input);
   sprintf(input, "%s.startingtree", base_fname);
   remove(input);
+  if(replicates > 0) {
+    sprintf(input, "%s.phy_phyml_boot_trees.txt", base_fname);
+    remove(input);
+    sprintf(input, "%s.phy_phyml_boot_stats.txt", base_fname);
+    remove(input);
+    }
   sprintf(input, "%s.phy_phyml_stats.txt", base_fname);
   in = fopen(input, "r");
   logL = 0;
@@ -1048,11 +1058,29 @@ void free_after_view_to_allseq(allseq *phyml_seqs)
 	free(phyml_seqs);
 }
 
+void initialize_random_series(void)
+{
+  static int first = TRUE;
+  unsigned seed;
+  if(first) {
+    first = FALSE;
+#ifdef WIN32
+    SYSTEMTIME timing;
+    GetSystemTime(&timing);
+    seed = (unsigned)(timing.wMilliseconds + 1000*(timing.wSecond + 60*(timing.wMinute + 60*(timing.wHour + 24*timing.wDay))));
+#else
+    struct tms timing;
+    seed = (unsigned)times(&timing);
+#endif
+    srand(seed);
+  }
+}
 
 void bootstrap_weights(allseq *seqs, int use_codons)
 {
 	phydbl buff;
 	int j, position;
+	initialize_random_series();
 	memset(seqs->wght, 0, seqs->clean_len * sizeof(int));
 	if(!use_codons) {//bootstrap sites
 		for(j = 0; j < seqs->clean_len; j++) {
@@ -1496,6 +1524,7 @@ void jumble_fct(char** seq, char** name, int notu)
 	int i, j, rando;
 	double buff;
 	char *tmp;
+	initialize_random_series();
 	for(i = 0; i<notu - 1; i++){
 		j = notu - i - 1;
 		buff  = rand();
@@ -1719,7 +1748,7 @@ char *parsimony_with_bootstrap(allseq *tmpallseqs, char **pars_seqs, char **pars
 			int maxbest, int replicates, Fl_Box *w_count, int *pinterrupted, int *preplicates_done)
 {
 	int i, j, steps, newsteps, count = 0, replicates_done = 0;
-	char *tree, *newtree, *p, *q;
+	char *tree, *newtree, *p;
 	
 	tree = parsimony_with_jumble(pars_seqs, pars_names, n_otu, njumbles, protein, 
 								 &steps, &count, maxbest);
@@ -1761,10 +1790,9 @@ char *parsimony_with_bootstrap(allseq *tmpallseqs, char **pars_seqs, char **pars
 			free(newtree);
 			i--;
 			if(i == 0) break;
-			q = strchr(p, '\n');
-			newtree = (char *)malloc(q - p + 1);
-			memcpy(newtree, p, q - p); newtree[q - p] = 0;
-			p = q + 1;
+			newtree = strdup(p+1);
+			free(tree);
+			tree = newtree;
 		}
 		++replicates_done;
 		free(tree);
